# Tower Capitalisation
#
# Aim: Given a tower refurbishment schedule for a category of towers, determine tower capitalisation, based on built sections.
#
#

ReportProgressInfo = False			# For debugging
ReportPreliminaryReports = False	# For initial development of program: reports data in different ways.
ReportTowerAllocation = False		# For listing tower / built sections when they're being allocated.


import operator
import sys
import os
import csv


CategorySpecs = {
1: {'Vmin': 66, 'Vmax':132, 'SingleCct': True, 'Grillage': True, 'Zone': 'C', 'Descript': 'Zone C - > 66 kV & < = 132 kV ; SINGLE CIRCUIT'},
2: {'Vmin': 66, 'Vmax':132, 'SingleCct': False, 'Grillage': True, 'Zone': 'B', 'Descript': 'Zone B - > 66 kV & < = 132 kV ; MULTIPLE CIRCUIT'},
3: {'Vmin': 66, 'Vmax':132, 'SingleCct': False, 'Grillage': True, 'Zone': 'C', 'Descript': 'Zone C - > 66 kV & < = 132 kV ; MULTIPLE CIRCUIT'},

4: {'Vmin': 66, 'Vmax':132, 'SingleCct': True, 'Grillage': False, 'Zone': 'B', 'Descript': 'Zone B - > 66 kV & < = 132 kV ; SINGLE CIRCUIT'},
5: {'Vmin': 66, 'Vmax':132, 'SingleCct': True, 'Grillage': False, 'Zone': 'C', 'Descript': 'Zone C - > 66 kV & < = 132 kV ; SINGLE CIRCUIT'},
6: {'Vmin': 66, 'Vmax':132, 'SingleCct': True, 'Grillage': False, 'Zone': 'D', 'Descript': 'Zone D - > 66 kV & < = 132 kV ; SINGLE CIRCUIT'},
7: {'Vmin': 132, 'Vmax':275, 'SingleCct': True, 'Grillage': False, 'Zone': 'B', 'Descript': 'Zone B - > 132 kV & < = 275 kV ; SINGLE CIRCUIT'},
8: {'Vmin': 132, 'Vmax':275, 'SingleCct': True, 'Grillage': False, 'Zone': 'C', 'Descript': 'Zone C - > 132 kV & < = 275 kV ; SINGLE CIRCUIT'},
9: {'Vmin': 132, 'Vmax':275, 'SingleCct': True, 'Grillage': False, 'Zone': 'D', 'Descript': 'Zone D - > 132 kV & < = 275 kV ; SINGLE CIRCUIT'},
10: {'Vmin': 275, 'Vmax':330, 'SingleCct': True, 'Grillage': False, 'Zone': 'B', 'Descript': 'Zone B - > 275 kV & < = 330 kV ; SINGLE CIRCUIT'},

11: {'Vmin': 66, 'Vmax':132, 'SingleCct': False, 'Grillage': False, 'Zone': 'B', 'Descript': 'Zone B - > 66 kV & < = 132 kV ; MULTIPLE CIRCUIT'},
12: {'Vmin': 66, 'Vmax':132, 'SingleCct': False, 'Grillage': False, 'Zone': 'C', 'Descript': 'Zone C - > 66 kV & < = 132 kV ; MULTIPLE CIRCUIT'},
13: {'Vmin': 66, 'Vmax':132, 'SingleCct': False, 'Grillage': False, 'Zone': 'D', 'Descript': 'Zone D - > 66 kV & < = 132 kV ; MULTIPLE CIRCUIT'},
14: {'Vmin': 132, 'Vmax':275, 'SingleCct': False, 'Grillage': False, 'Zone': 'B', 'Descript': 'Zone B - > 132 kV & < = 275 kV ; MULTIPLE CIRCUIT'},
15: {'Vmin': 132, 'Vmax':275, 'SingleCct': False, 'Grillage': False, 'Zone': 'C', 'Descript': 'Zone C - > 132 kV & < = 275 kV ; MULTIPLE CIRCUIT'},
16: {'Vmin': 132, 'Vmax':275, 'SingleCct': False, 'Grillage': False, 'Zone': 'D', 'Descript': 'Zone D - > 132 kV & < = 275 kV ; MULTIPLE CIRCUIT'},
17: {'Vmin': 275, 'Vmax':330, 'SingleCct': False, 'Grillage': False, 'Zone': 'B', 'Descript': 'Zone B - > 275 kV & < = 330 kV ; MULTIPLE CIRCUIT'},
18: {'Vmin': 275, 'Vmax':330, 'SingleCct': False, 'Grillage': False, 'Zone': 'C', 'Descript': 'Zone C - > 275 kV & < = 330 kV ; MULTIPLE CIRCUIT'}
}
		
	

csvfile = open('Input_TowerData.csv', 'r')

TowerData = {}

try:
	reader = csv.DictReader(csvfile)
	for row in reader:
		keyID = row['FuncLoc']		
			
		tmpDict = {
			'BS': int(row['BS']),
			'Year': int(row['Year']),
			'MultCct': (row['IsMulti'] == "TRUE"),
			'Voltage': int(row['Voltage']),
			'Grillage': (row['IsGrillage'] == "TRUE"),
			'CorrosionZone': (row['CorrosionZone'])
		}
				
		HaveNotRefurb = (row['AlreadyRefurb'] == "FALSE")
		
		# Include tower if not already refurbed
		if HaveNotRefurb:
			TowerData[keyID] = tmpDict
			
finally:
	csvfile.close()
	
	
		
	
	

	
#
#
# OK, read in the tower refurbishment data
#
#

EarliestYearOfRefurb = 9999
LatestYearOfRefurb = 0


csvfile = open('Input_TowerRefurbProjection.csv', 'r')

RefurbData = {}

try:
	reader = csv.DictReader(csvfile)
	
	for row in reader:
		keyID = int(row['ID'])
		
		tmpDict = {}
		for k in row.keys():
			if k.isdigit():	# Columns that are years only.
				EarliestYearOfRefurb = min(EarliestYearOfRefurb, int(k))
				LatestYearOfRefurb = max(LatestYearOfRefurb, int(k))
				tmpDict[int(k)] = float(row[k])    # Assume column headings are years (integers), and the refurb value is real
			
		RefurbData[keyID] = tmpDict
		
finally:
	csvfile.close()
	




	
if ReportProgressInfo:
	print("\nTower RefurbData Data")
	print(RefurbData.keys())
	print(RefurbData)
		

		
	


#
#
# Determine a category for each tower
#
#

for tower in TowerData:
	myCategory = 0
	
	towerVolts = TowerData[tower]['Voltage']
	towerMultCct = TowerData[tower]['MultCct']
	towerGrillage = TowerData[tower]['Grillage']
	towerCorrosion = TowerData[tower]['CorrosionZone']
	
	# For this tower, go through the categories, until the relevant category is found.
	for cat in CategorySpecs:
		if (
			(towerGrillage == CategorySpecs[cat]['Grillage']) and 
			(towerCorrosion == CategorySpecs[cat]['Zone']) and
			(towerVolts <= CategorySpecs[cat]['Vmax']) and
			(towerVolts > CategorySpecs[cat]['Vmin']) 
			):
				
			if (CategorySpecs[cat]['SingleCct'] and (not towerMultCct)): 	
				myCategory = cat
				break
			elif ((not CategorySpecs[cat]['SingleCct']) and (towerMultCct)):
				myCategory = cat
				break
		
	
	if (myCategory == 0):
		print("WARNING - DATA ERROR - category")
		print("COULD NOT ASSIGN A CATEGORY. CHECK TOWER DATA. CHECK CATEGORY DATA.")
		print(tower)
	TowerData[tower]['Category'] = myCategory
	
	
	
if ReportProgressInfo:
	print("TowerData is below")
	print(TowerData.keys())	
	print(TowerData)


	
	
#
#
# For a given combination of tower category, built section, year of tower commissioning, determine total tower numbers
#
#

TowerCount = {}

for towerIdx in TowerData:
	tower = TowerData[towerIdx]
	
	mytuple = (tower['BS'], tower['Year'], tower['Category'])
		
	myVal = TowerCount.get(mytuple, 0)
		
	TowerCount[mytuple] = myVal + 1
		
if ReportProgressInfo:
	print("\nTowerCount Data")
	print(TowerCount.keys())
	print(TowerCount)




#
#
# Determine, for a given category / BS, the earliest year of a tower establishment
# eg: EarliestYearOfRefurbForGivenCatBS[ (cat, BS) ] = 1963
#
#

EarliestYearOfRefurbForGivenCatBS = {}

for tuple in TowerCount:
	BS = tuple[0]
	Year = tuple[1]
	Cat = tuple[2]
	
	duple = (Cat, BS)
	
	mytmp = EarliestYearOfRefurbForGivenCatBS.get(duple, 9999)
	EarliestYearOfRefurbForGivenCatBS[duple] = min(mytmp, Year)
	











#
#
#  For a given tower category and built section, determine all years of commissioning (and their totals), and the grand total
#
#

TowerCountTotals = {}

for tuple in TowerCount:
	BS = tuple[0]
	Year = tuple[1]
	Cat = tuple[2]
	myTotal = TowerCount[tuple]
	
	
	tmpDict = TowerCountTotals.get( (Cat, BS), {})
	
	tmpValTotal = tmpDict.get('Total', 0)
	
	tmpDict['Total'] = tmpValTotal + myTotal
	tmpDict[Year] = myTotal
	
	TowerCountTotals[ (Cat, BS) ] = tmpDict
	
	
if ReportProgressInfo:	
	print("\nTowerCountTotals Data")
	print(TowerCountTotals.keys())
	print(TowerCountTotals)








if ReportTowerAllocation:
	print("\n\nSTART OF MAIN TOWER ALLOCATION\n\n")
		

#
# Now go through each category.
# - First, determine an order to process each category / tower built section grouping.
# - Second, use this order, and assign as per the refurb schedule
#
#		

TowerProcessOrder = []
TowerProcessOrderCount = []
TowerProcessOrderBS = []

LastYearOfRefurb = {}	# for a given BS, category tuple, what is the first and last year of rebuild.
FirstYearOfRefurb = {}
TowersRefurbed = {}
TowersRefurbedDetailed = {}

TowersNotDone = {}
TowersNotDoneFinalBS = {}
TowersDoneInFinalYearBSNotComplete = {}

OrderOfBSForACategory = {}  # Keys are Category IDs, each value is a list of BS's in order that they are applied.



for catID in CategorySpecs:
	TowerProcessOrder = []
	
	for towerGroupIdx in TowerCount:  # the index is the tuple
		if (towerGroupIdx[2] == catID):
			TowerProcessOrder.append( (towerGroupIdx[0], towerGroupIdx[1], TowerCount[towerGroupIdx]) )    # BS, Year, Count
			
	# Determine the order that BS's should be processed in, for this category.
	# Sort in order of oldest first. Then sort by smallest for BS's with the same year.
	TowerProcessOrder = sorted(TowerProcessOrder,key=lambda x: (x[1], x[2]))
	
	if ReportTowerAllocation:
		print("Category: " + str(catID))
		print("TowerProcessOrder")
		print(TowerProcessOrder)
	
		
	
	for currentYear in sorted(RefurbData[catID]):
		if ReportProgressInfo:
			print("currentYear" + str(currentYear))
			print(RefurbData[catID][currentYear])
		
		# Number of towers available to allocate to a BS.
		TowersLeftThisYear = RefurbData[catID][currentYear]
		
		# Are we in the final year?
		FinalYear = (currentYear == max(RefurbData[catID]))
		
		
		for curIdx in range(0, len(TowerProcessOrder)):
		
			if ReportProgressInfo:
				print(TowersLeftThisYear)
				#print(curIdx)
				#	print(TowerProcessOrder)
			
			tmp = TowerProcessOrder[curIdx]
			BS = tmp[0]
			Year = tmp[1]
			Count = tmp[2]
			
			if (Count > 0):
				# We should have more than 0 towers to refurb in a given category!
				# We may have 0 if we've already processed a BS.
				
				if ReportProgressInfo:
					print(BS)
					print(Year)
					print(Count)
				
				tmpList = OrderOfBSForACategory.get(catID, [])
				if BS not in tmpList:
					tmpList.append(BS)
				OrderOfBSForACategory[catID] = tmpList
		
		
									
				if not ((catID, BS) in FirstYearOfRefurb.keys()):
					FirstYearOfRefurb[ (catID, BS) ] = currentYear
						
						
				if (Count <= TowersLeftThisYear):
					# There's fewer towers in this BS then are available this year.
					# So we can refurb them all.
				
					LastYearOfRefurb[(catID, BS)] = currentYear
					
					if ReportProgressInfo:
						print("abbbbbc")
						#print(type(Count))
						#print(Count)
						#print(type(TowersLeftThisYear))
						#print(TowersLeftThisYear)
					
					# Reduce the number of towers left for other BS's
					TowersLeftThisYear = TowersLeftThisYear - Count
					
					# Set the number of towers required for this BS to 0.
					TowerProcessOrder[curIdx] = (BS, Year, 0)
					
					tmpVal = TowersRefurbed.get( (catID, BS), 0)
					TowersRefurbed[(catID, BS)] = tmpVal + Count
					
					tmpDict = TowersRefurbedDetailed.get((catID, BS), {})
					TowersRefurbedDetailed[(catID, BS)] = tmpDict
					TowersRefurbedDetailed[(catID, BS)][currentYear] = Count
					
					
				else:
					# There's more towers than we can do this year. Do as many as possible.
					TowerProcessOrder[curIdx] = (BS, Year, (Count - TowersLeftThisYear))
					#FirstYearOfRefurb[ (catID, BS) ] = currentYear

					
					tmpVal = TowersRefurbed.get( (catID, BS), 0)
					TowersRefurbed[(catID, BS)] = tmpVal + TowersLeftThisYear
					
				
					if (TowersLeftThisYear <> 0):  # Want to exclude all the zero's printing when the frist year we do anything is not for a while
						tmpDict = TowersRefurbedDetailed.get((catID, BS), {})
						TowersRefurbedDetailed[(catID, BS)] = tmpDict
						TowersRefurbedDetailed[(catID, BS)][currentYear] = TowersLeftThisYear
					
					
					if FinalYear:
						TowersNotDone[catID] = (Count - TowersLeftThisYear)
						TowersNotDoneFinalBS[catID] = BS
						TowersDoneInFinalYearBSNotComplete[catID] = TowersRefurbed[(catID, BS)]  #TowersLeftThisYear
						#TowersRefurbedDetailed[(catID, BS)]['TowersDoneBSForIncompleteBS'] = TowersLeftThisYear
						#TowersRefurbedDetailed[(catID, BS)]['TowersNotDone'] = (Count - TowersLeftThisYear)
					
					
					TowersLeftThisYear = 0
					
					break
					
if ReportTowerAllocation:
	print("\n\nEND OF MAIN TOWER ALLOCATION\n\n")					
					
if ReportProgressInfo:				
	print("FirstYearOfRefurb")	
	print(FirstYearOfRefurb)
	sorted_x = sorted(FirstYearOfRefurb.items(), key=operator.itemgetter(1))
	for i in sorted_x:
		print(i)
						
	print("LastYearOfRefurb")	
	print(LastYearOfRefurb)
	sorted_x = sorted(LastYearOfRefurb.items(), key=operator.itemgetter(1))
	for i in sorted_x:
		print(i)
		
	print("TowersRefurbedDetailed")
	print(TowersRefurbedDetailed)	
	sorted_x = sorted(TowersRefurbedDetailed.items(), key=operator.itemgetter(1))
	for i in sorted_x:
		print(i)
		


	
	
	
	
#
#
# OK, process the data some more
#
# for each category, determine the total number of towers that finished in the last year
#
#

CategoryTotalTowersCompletedByYear = {}


#for curIdx in CategorySpecs:
#
#for (curIdx, BS) in sorted(TowersRefurbedDetailed):
for (curIdx, BS) in sorted(LastYearOfRefurb):
	#fwcsv.writerow([curIdx, CategorySpecs[curIdx]['Descript'], BS, FirstYearOfRefurb[(curIdx, BS)], LastYearOfRefurb[(curIdx, BS)], TowersRefurbed[(curIdx, BS)]])
		
	#print(curIdx)
	#print(BS)
	#print(tmpCatidx)
	
	tmpCatDict = CategoryTotalTowersCompletedByYear.get(curIdx, {})
	year = LastYearOfRefurb.get( (curIdx, BS), 0)
	towers = TowersRefurbed[(curIdx, BS)]
	
	tmpVal = tmpCatDict.get(year, 0)
	totVal = tmpCatDict.get('Total', 0)
	
	
	tmpCatDict[year] = tmpVal + towers
	tmpCatDict['Total'] = totVal + towers
	
	CategoryTotalTowersCompletedByYear[curIdx] = tmpCatDict
	
	
if ReportProgressInfo:
	print("CategoryTotalTowersCompletedByYear")
	print(CategoryTotalTowersCompletedByYear)
	
			
			
			
	
	
	
	
	
#
#
# OK, Now output data
#
#






# 
# Firstly, output, for all categories, for all BS's the year that each number of towers is due.
#
#
# Basically, a representation of TowerCount data
#

#
# Get a list of all years (in the past) that had towers commissioned in that year. NB: this will typically have 'gaps', especially earlier on in the profile where there are no more towers
# left of that particular year. Eg: 1960.
#

ListOfAllYears = []

for towerTuple in TowerCount.keys():

	ListOfAllYears.append(towerTuple[1])
	

ListOfAllYears = sorted(list(set(ListOfAllYears)))
ListOfAllYearsTotal = sorted(list(set(ListOfAllYears)))
ListOfAllYearsTotal.append("Total")

if ReportProgressInfo:
	print("ListOfAllYears")
	print(ListOfAllYears)

	print("ListOfAllYearsTotal")
	print(ListOfAllYearsTotal)



ListOfAllBS = sorted(list(set(x[1] for x in TowerCountTotals)))

if ReportProgressInfo:
	print(ListOfAllBS)


if ReportPreliminaryReports:



	fw = open('Output_1a_Category_Tower_BS_Year_Data_SortedByBS.csv', 'wb')

	fw.write("CatID,Category,BS,")

	for i in ListOfAllYears:
		fw.write(str(i) + ",")
	fw.write("Total\n")

	writer = csv.DictWriter(fw, fieldnames=ListOfAllYearsTotal)

	for curIdx in sorted(CategorySpecs):


		#tmpDict['Total'] = tmpValTotal + myTotal
		#tmpDict[Year] = myTotal
		#TowerCountTotals[ (Cat, BS) ] = tmpDict
		
		for BS in ListOfAllBS:
		#for tuple in TowerCountTotals.keys():
		#	if (tuple[0] == curIdx):
				# Match of categories
		
			if (curIdx, BS) in TowerCountTotals.keys():

				#BS = tuple[1]
				InfoToPrint = TowerCountTotals[(curIdx, BS)]
				
				fw.write(
					str(curIdx) + "," + 
					str(CategorySpecs[curIdx]['Descript']) + "," + 
					str(BS) + ","
					)

				writer.writerow(InfoToPrint)		
	fw.close()





	fw = open('Output_1b_Category_Tower_BS_Year_Data_SortedByYear.csv', 'wb')

	fw.write("CatID,Category,BS,")

	for i in ListOfAllYears:
		fw.write(str(i) + ",")
	fw.write("Total\n")

	writer = csv.DictWriter(fw, fieldnames=ListOfAllYearsTotal)

	for curIdx in sorted(CategorySpecs):

		ListOfCatBSCombosDone = []

		#tmpDict['Total'] = tmpValTotal + myTotal
		#tmpDict[Year] = myTotal
		#TowerCountTotals[ (Cat, BS) ] = tmpDict
		
		for year in ListOfAllYears:
		#	print("year")
		#	print(year)
		#	#########################
			for BS in ListOfAllBS:
			#for tuple in TowerCountTotals.keys():
			#	if (tuple[0] == curIdx):
					# Match of categories
			
				if (curIdx, BS) in TowerCountTotals.keys():
					
					if year in TowerCountTotals[(curIdx, BS)]:
					#	print("ListOfCatBSCombosDone")
					#	print(ListOfCatBSCombosDone)
						if not ((curIdx, BS) in ListOfCatBSCombosDone):

							
							#BS = tuple[1]
							InfoToPrint = TowerCountTotals[(curIdx, BS)]
							
							fw.write(
								str(curIdx) + "," + 
								str(CategorySpecs[curIdx]['Descript']) + "," + 
								str(BS) + ","
								)

							writer.writerow(InfoToPrint)		
							
							ListOfCatBSCombosDone.append( (curIdx, BS))
	fw.close()







	fw = open('Output_1c_Category_Tower_BS_Year_Data_SortedByBSThenCategory.csv', 'wb')

	fw.write("CatID,Category,BS,")

	for i in ListOfAllYears:
		fw.write(str(i) + ",")
	fw.write("Total\n")

	writer = csv.DictWriter(fw, fieldnames=ListOfAllYearsTotal)


	for BS in ListOfAllBS:
		for curIdx in sorted(CategorySpecs):

		#tmpDict['Total'] = tmpValTotal + myTotal
		#tmpDict[Year] = myTotal
		#TowerCountTotals[ (Cat, BS) ] = tmpDict
		

		#for tuple in TowerCountTotals.keys():
		#	if (tuple[0] == curIdx):
				# Match of categories
		
			if (curIdx, BS) in TowerCountTotals.keys():

				#BS = tuple[1]
				InfoToPrint = TowerCountTotals[(curIdx, BS)]
				
				fw.write(
					str(BS) + "," +
					str(curIdx) + "," + 
					str(CategorySpecs[curIdx]['Descript']) + "," 
					
					)

				writer.writerow(InfoToPrint)		
	fw.close()











	#
	# Secondly, output, for all categories, for all BS's the first and last year, and the tower count
	#

	fw = open('Output_2_Category_BS_Information.csv', 'wb')
	fwcsv = csv.writer(fw) #, newline = '')

	fw.write("CatID,Category,BS,FirstYear,LastYear,TotalRefurbed,TowersDoneBSForIncompleteBS,TowersNotDone,Info\n")


	#OrderOfBSForACategory[curIdx] = tmpList
	#ListOfYearsForTowerWorkHeadings = ListOfYearsForTowerWork + ["Total"]


	if ReportProgressInfo:
		print("OrderOfBSForACategory")
		print(OrderOfBSForACategory)

	for curIdx in CategorySpecs:
		#for (tmpCatidx, BS) in sorted(FirstYearOfRefurb):
		if curIdx in OrderOfBSForACategory:
			
			for BS in OrderOfBSForACategory[curIdx]:
		
				if (TowersNotDoneFinalBS.get(curIdx,"") == BS):
					TowersRemaining = TowersNotDone[curIdx]	
					TowersDone = TowersDoneInFinalYearBSNotComplete[curIdx]
				else:
					TowersRemaining = "NA"
					TowersDone = "NA"
				
								
				
				fw.write(
					str(curIdx) + "," + 
					str(CategorySpecs[curIdx]['Descript']) + "," + 
					str(BS) + "," + 
					str(FirstYearOfRefurb.get((curIdx, BS), "NA3")) + "," + 
					str(LastYearOfRefurb.get((curIdx, BS), "NA")) + "," + 
					str(TowersRefurbed.get((curIdx, BS), "NA2")) + "," +
					str(TowersDone) + "," +
					str(TowersRemaining) + ","
					
									
					)
				
				if ( curIdx, BS) in TowersRefurbedDetailed:
					fw.write("'")
					for tmpIdx in sorted(TowersRefurbedDetailed[(curIdx, BS)]):
						fw.write(str(tmpIdx) + ": %.2f; " % TowersRefurbedDetailed[(curIdx, BS)][tmpIdx])
				else:
					fw.write("We haven't refurbed any towers of this type!" + str( (curIdx, BS)))
				
				
				fw.write("\n")
					
					
		else:
			fw.write( str(curIdx) + "," + 
						str(CategorySpecs[curIdx]['Descript']) + "," + 
						"No BS's available\n" )
				
			
			
	fw.close()











if ReportProgressInfo:
	print("EarliestYearOfRefurb")
	print(EarliestYearOfRefurb)
	print(LatestYearOfRefurb)

ListOfYearsForTowerWork = range(EarliestYearOfRefurb, LatestYearOfRefurb + 1)
ListOfYearsForTowerWorkHeadings = ListOfYearsForTowerWork + ["Total"]

if ReportProgressInfo:
	print(ListOfYearsForTowerWork)
	print(ListOfYearsForTowerWorkHeadings)

#
# Thirdly, output, for all categories, the final year that we finish the towers in a built section.
# This is what goes into the repex model.
#


fw = open('Output_Category_TowersComplete_ByYear.csv', 'wb')

fw.write("CatID,Category,")

for i in ListOfYearsForTowerWorkHeadings :
	fw.write(str(i) + ",")
fw.write("TowersCompleteButBSNotFinished,TowersRemaining\n")

writer = csv.DictWriter(fw, fieldnames=ListOfYearsForTowerWorkHeadings, lineterminator='')

for curIdx in sorted(CategorySpecs):
	tmpDict = CategoryTotalTowersCompletedByYear.get(curIdx, {})

	fw.write(
		str(curIdx) + "," + 
		str(CategorySpecs[curIdx]['Descript']) + ","
		)

	writer.writerow(tmpDict)	
	
	fw.write("," + str(TowersDoneInFinalYearBSNotComplete.get(curIdx, "")) + "," + str(TowersNotDone.get(curIdx,"")) + "\n")

	
fw.close()







if ReportPreliminaryReports:

	#
	# Output, for all categories, the final year that we finish the towers in a built section. - cumulative total.
	#

	CategoryTotalTowersCompletedByYearCumulative = {}

	for curIdx in sorted(CategorySpecs):
		
		if curIdx in CategoryTotalTowersCompletedByYear:
			if ReportProgressInfo:
				print("Info is below!")
				print(CategoryTotalTowersCompletedByYear)
				print(curIdx)
				print(CategoryTotalTowersCompletedByYear[curIdx])
				print(ListOfYearsForTowerWork)
				print(ListOfYearsForTowerWork[0])
			
			
			# Get the first value for the first year.
			firstYear = ListOfYearsForTowerWork[0]
			
			tmpDict = {
			firstYear: CategoryTotalTowersCompletedByYear[curIdx].get(firstYear, 0)
			}
			
			CategoryTotalTowersCompletedByYearCumulative[curIdx] = tmpDict
			
			tmpDict = CategoryTotalTowersCompletedByYear.get(curIdx, {})
			
			if ReportProgressInfo:
				print(ListOfYearsForTowerWork)
				print(max(ListOfYearsForTowerWork))
			
			for year in range(firstYear + 1, max(ListOfYearsForTowerWork) + 1):
				CategoryTotalTowersCompletedByYearCumulative[curIdx][year] = CategoryTotalTowersCompletedByYearCumulative[curIdx][year - 1] + CategoryTotalTowersCompletedByYear[curIdx].get(year, 0)
			
			
		
		
			
			
	CategoryTotalTowersCompletedActualTowersByYearCumulative = {}


	for curIdx in sorted(CategorySpecs):
		
		if curIdx in RefurbData:
			if ReportProgressInfo:
				print("Info is below!")
				print(RefurbData)
				print(curIdx)
				print(RefurbData[curIdx])
				print(ListOfYearsForTowerWork)
				print(ListOfYearsForTowerWork[0])
			
			
			# Get the first value for the first year.
			firstYear = ListOfYearsForTowerWork[0]
			
			tmpDict = {
			firstYear: RefurbData[curIdx].get(firstYear, 0)
			}
			
			CategoryTotalTowersCompletedActualTowersByYearCumulative[curIdx] = tmpDict
			
			tmpDict = CategoryTotalTowersCompletedByYear.get(curIdx, {})
			
			if ReportProgressInfo:
				print(ListOfYearsForTowerWork)
				print(max(ListOfYearsForTowerWork))
			
			for year in range(firstYear + 1, max(ListOfYearsForTowerWork) + 1):
				CategoryTotalTowersCompletedActualTowersByYearCumulative[curIdx][year] = CategoryTotalTowersCompletedActualTowersByYearCumulative[curIdx][year - 1] + RefurbData[curIdx].get(year, 0)
			
			

	for curIdx in CategorySpecs:
		lastyear = max(ListOfYearsForTowerWork) 
		
		if (CategoryTotalTowersCompletedActualTowersByYearCumulative[curIdx].get(lastyear, 0) == 0):
			CategorySpecs[curIdx]['HasTowersAtAll'] = False
		else:
			CategorySpecs[curIdx]['HasTowersAtAll'] = True
		





	fw = open('Output_4a_Category_TowersComplete_ByYear_Cumulative.csv', 'wb')

	fw.write("CatID,Category,")

	for i in ListOfYearsForTowerWork :
		fw.write(str(i) + ",")
	fw.write("\n")

	writer = csv.DictWriter(fw, fieldnames=ListOfYearsForTowerWork, lineterminator='')

	for curIdx in sorted(CategorySpecs):
		tmpDict = CategoryTotalTowersCompletedByYearCumulative.get(curIdx, {})

		fw.write(
			str(curIdx) + "," + 
			str(CategorySpecs[curIdx]['Descript']) + ","
			)
		
		if ReportProgressInfo:	
			print(ListOfAllYearsTotal)
			print(tmpDict.keys())
			print(tmpDict.values())

		writer.writerow(tmpDict)	
		
		#fw.write("," + str(TowersDoneInFinalYearBSNotComplete.get(curIdx, "")) + "," + str(TowersNotDone.get(curIdx,"")) + "\n")
		fw.write("\n")

		
	fw.close()






	fw = open('Output_4b_Category_TowersComplete_ByYear_Cumulative_TowersCompleted.csv', 'wb')

	fw.write("CatID,Category,")

	for i in ListOfYearsForTowerWork :
		fw.write(str(i) + ",")
	fw.write("\n")

	writer = csv.DictWriter(fw, fieldnames=ListOfYearsForTowerWork, lineterminator='')

	for curIdx in sorted(CategorySpecs):
		tmpDict = CategoryTotalTowersCompletedActualTowersByYearCumulative.get(curIdx, {})

		fw.write(
			str(curIdx) + "," + 
			str(CategorySpecs[curIdx]['Descript']) + ","
			)
			
		writer.writerow(tmpDict)	
		
		#fw.write("," + str(TowersDoneInFinalYearBSNotComplete.get(curIdx, "")) + "," + str(TowersNotDone.get(curIdx,"")) + "\n")
		fw.write("\n")
		
		
		
	fw.close()






	fw = open('Output_4c_Category_TowersComplete_ByYear_Cumulative_Combined.csv', 'wb')

	fw.write("Type_CatID_Category,")

	for i in ListOfYearsForTowerWork :
		fw.write(str(i) + ",")
	fw.write("\n")

	writer = csv.DictWriter(fw, fieldnames=ListOfYearsForTowerWork, lineterminator='')

	for curIdx in sorted(CategorySpecs):

		if curIdx in CategoryTotalTowersCompletedByYearCumulative:
			tmpDict = CategoryTotalTowersCompletedByYearCumulative.get(curIdx, {})

			fw.write(
				"BS_" + 
				str(curIdx) + "_" + 
				str(CategorySpecs[curIdx]['Descript']) + ","
				)
			
			writer.writerow(tmpDict)	
			fw.write("\n")

		
		if CategorySpecs[curIdx]['HasTowersAtAll']:
			tmpDict = CategoryTotalTowersCompletedActualTowersByYearCumulative.get(curIdx, {})

			fw.write(
				"Towers_" + 
				str(curIdx) + "_" + 
				str(CategorySpecs[curIdx]['Descript']) + ","
				)
				
			writer.writerow(tmpDict)	
			fw.write("\n")
